@@ -33,6 +33,7 @@ from openhands.runtime.base import (
     RuntimeDisconnectedError,
     RuntimeNotFoundError,
     RuntimeNotReadyError,
+    RuntimeUnavailableError,
 )
 from openhands.runtime.builder.remote import RemoteRuntimeBuilder
 from openhands.runtime.plugins import PluginRequirement
@@ -235,6 +236,7 @@ class RemoteRuntime(Runtime):
             self.config.sandbox.user_id,
             plugin_args,
             browsergym_args,
+            is_root=not self.config.run_as_openhands,  # is_root=True when running as root
         )
         start_request = {
             'image': self.container_image,
@@ -245,17 +247,21 @@ class RemoteRuntime(Runtime):
         }
 
         # Start the sandbox using the /start endpoint
-        with self._send_request(
-            'POST',
-            f'{self.config.sandbox.remote_runtime_api_url}/start',
-            is_retry=False,
-            json=start_request,
-        ) as response:
-            self._parse_runtime_response(response)
-        self.log(
-            'debug',
-            f'Runtime started. URL: {self.runtime_url}',
-        )
+        try:
+            with self._send_request(
+                'POST',
+                f'{self.config.sandbox.remote_runtime_api_url}/start',
+                is_retry=False,
+                json=start_request,
+            ) as response:
+                self._parse_runtime_response(response)
+            self.log(
+                'debug',
+                f'Runtime started. URL: {self.runtime_url}',
+            )
+        except requests.HTTPError as e:
+            self.log('error', f'Unable to start runtime: {e}')
+            raise RuntimeUnavailableError() from e
 
     def _resume_runtime(self):
         with self._send_request(
@@ -330,13 +336,13 @@ class RemoteRuntime(Runtime):
         assert 'runtime_id' in runtime_data
         assert runtime_data['runtime_id'] == self.runtime_id
         assert 'pod_status' in runtime_data
-        pod_status = runtime_data['pod_status']
+        pod_status = runtime_data['pod_status'].lower()
         self.log('debug', f'Pod status: {pod_status}')
 
         # FIXME: We should fix it at the backend of /start endpoint, make sure
         # the pod is created before returning the response.
         # Retry a period of time to give the cluster time to start the pod
-        if pod_status == 'Ready':
+        if pod_status == 'ready':
             try:
                 with self._send_request(
                     'GET',
@@ -352,14 +358,14 @@ class RemoteRuntime(Runtime):
                 )
             return
         elif (
-            pod_status == 'Not Found'
-            or pod_status == 'Pending'
-            or pod_status == 'Running'
+            pod_status == 'not found'
+            or pod_status == 'pending'
+            or pod_status == 'running'
         ):  # nb: Running is not yet Ready
             raise RuntimeNotReadyError(
                 f'Runtime (ID={self.runtime_id}) is not yet ready. Status: {pod_status}'
             )
-        elif pod_status in ('Failed', 'Unknown'):
+        elif pod_status in ('failed', 'unknown', 'crashloopbackoff'):
             # clean up the runtime
             self.close()
             raise RuntimeError(
